/*
 * 				Twidere - Twitter client for Android
 * 
 *  Copyright (C) 2012-2014 Mariotaku Lee <mariotaku.lee@gmail.com>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.mariotaku.twidere.model;

import org.mariotaku.twidere.provider.TweetStore.Accounts;

import android.content.ContentResolver;
import android.content.Context;
import android.database.Cursor;
import android.graphics.Color;
import android.os.Parcel;
import android.os.Parcelable;

public class Account implements Parcelable {

	public static final Parcelable.Creator<Account> CREATOR = new Parcelable.Creator<Account>() {

		@Override
		public Account createFromParcel(final Parcel in) {
			return new Account(in);
		}

		@Override
		public Account[] newArray(final int size) {
			return new Account[size];
		}
	};

	public final String screen_name, name, profile_image_url, profile_banner_url;

	public final long account_id;

	public final int color;

	public final boolean is_activated;
	public final boolean is_dummy;

	public Account(final Cursor cursor, final Indices indices) {
		is_dummy = false;
		screen_name = indices.screen_name != -1 ? cursor.getString(indices.screen_name) : null;
		name = indices.name != -1 ? cursor.getString(indices.name) : null;
		account_id = indices.account_id != -1 ? cursor.getLong(indices.account_id) : -1;
		profile_image_url = indices.profile_image_url != -1 ? cursor.getString(indices.profile_image_url) : null;
		profile_banner_url = indices.profile_banner_url != -1 ? cursor.getString(indices.profile_banner_url) : null;
		color = indices.color != -1 ? cursor.getInt(indices.color) : Color.TRANSPARENT;
		is_activated = indices.is_activated != -1 ? cursor.getInt(indices.is_activated) == 1 : false;
	}

	public Account(final Parcel source) {
		is_dummy = source.readInt() == 1;
		is_activated = source.readInt() == 1;
		account_id = source.readLong();
		name = source.readString();
		screen_name = source.readString();
		profile_image_url = source.readString();
		profile_banner_url = source.readString();
		color = source.readInt();
	}

	@Override
	public int describeContents() {
		return 0;
	}

	@Override
	public String toString() {
		return "Account{screen_name=" + screen_name + ", name=" + name + ", profile_image_url=" + profile_image_url
				+ ", profile_banner_url=" + profile_banner_url + ", account_id=" + account_id + ", user_color=" + color
				+ ", is_activated=" + is_activated + "}";
	}

	@Override
	public void writeToParcel(final Parcel out, final int flags) {
		out.writeInt(is_dummy ? 1 : 0);
		out.writeInt(is_activated ? 1 : 0);
		out.writeLong(account_id);
		out.writeString(name);
		out.writeString(screen_name);
		out.writeString(profile_image_url);
		out.writeString(profile_banner_url);
		out.writeInt(color);
	}

	public static Account getAccount(final Context context, final long account_id) {
		if (context == null) return null;
		final ContentResolver cr = context.getContentResolver();
		final Cursor cur = cr.query(Accounts.CONTENT_URI, Accounts.COLUMNS, Accounts.ACCOUNT_ID + " = " + account_id,
				null, null);
		if (cur == null) return null;
		try {
			if (cur.getCount() > 0 && cur.moveToFirst()) {
				final Indices indices = new Indices(cur);
				cur.moveToFirst();
				return new Account(cur, indices);
			}
			return null;
		} finally {
			cur.close();
		}
	}

	public static final class Indices {

		public final int screen_name, name, account_id, profile_image_url, profile_banner_url, color, is_activated,
				auth_type;

		public Indices(final Cursor cursor) {
			screen_name = cursor.getColumnIndex(Accounts.SCREEN_NAME);
			name = cursor.getColumnIndex(Accounts.NAME);
			account_id = cursor.getColumnIndex(Accounts.ACCOUNT_ID);
			profile_image_url = cursor.getColumnIndex(Accounts.PROFILE_IMAGE_URL);
			profile_banner_url = cursor.getColumnIndex(Accounts.PROFILE_BANNER_URL);
			color = cursor.getColumnIndex(Accounts.COLOR);
			is_activated = cursor.getColumnIndex(Accounts.IS_ACTIVATED);
			auth_type = cursor.getColumnIndex(Accounts.AUTH_TYPE);
		}

		@Override
		public String toString() {
			return "Indices{screen_name=" + screen_name + ", name=" + name + ", account_id=" + account_id
					+ ", profile_image_url=" + profile_image_url + ", profile_banner_url=" + profile_banner_url
					+ ", user_color=" + color + ", is_activated=" + is_activated + "}";
		}
	}

}
