/*
 * 				Twidere - Twitter client for Android
 * 
 *  Copyright (C) 2012-2014 Mariotaku Lee <mariotaku.lee@gmail.com>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.mariotaku.twidere.model;

import java.io.Serializable;
import java.util.Comparator;

import android.os.Parcel;
import android.os.Parcelable;

public class ParcelableDirectMessage implements Parcelable, Serializable, Comparable<ParcelableDirectMessage> {

	private static final long serialVersionUID = -3721836808981416526L;

	public static final Parcelable.Creator<ParcelableDirectMessage> CREATOR = new Parcelable.Creator<ParcelableDirectMessage>() {
		@Override
		public ParcelableDirectMessage createFromParcel(final Parcel in) {
			return new ParcelableDirectMessage(in);
		}

		@Override
		public ParcelableDirectMessage[] newArray(final int size) {
			return new ParcelableDirectMessage[size];
		}
	};

	public static final Comparator<ParcelableDirectMessage> MESSAGE_ID_COMPARATOR = new Comparator<ParcelableDirectMessage>() {

		@Override
		public int compare(final ParcelableDirectMessage object1, final ParcelableDirectMessage object2) {
			final long diff = object2.id - object1.id;
			if (diff > Integer.MAX_VALUE) return Integer.MAX_VALUE;
			if (diff < Integer.MIN_VALUE) return Integer.MIN_VALUE;
			return (int) diff;
		}
	};

	public final long account_id, id, timestamp;

	public final long sender_id, recipient_id;
	public final boolean is_outgoing;

	public final String text_html, text_plain, text_unescaped;

	public final String sender_name, recipient_name, sender_screen_name, recipient_screen_name;
	public final String sender_profile_image_url, recipient_profile_image_url;

	public ParcelableDirectMessage(final Parcel in) {
		account_id = in.readLong();
		id = in.readLong();
		timestamp = in.readLong();
		sender_id = in.readLong();
		recipient_id = in.readLong();
		is_outgoing = in.readInt() == 1;
		text_html = in.readString();
		text_plain = in.readString();
		sender_name = in.readString();
		recipient_name = in.readString();
		sender_screen_name = in.readString();
		recipient_screen_name = in.readString();
		sender_profile_image_url = in.readString();
		recipient_profile_image_url = in.readString();
		text_unescaped = in.readString();
	}

	@Override
	public int compareTo(final ParcelableDirectMessage another) {
		if (another == null) return 0;
		final long diff = another.id - id;
		if (diff > Integer.MAX_VALUE) return Integer.MAX_VALUE;
		if (diff < Integer.MIN_VALUE) return Integer.MIN_VALUE;
		return (int) diff;
	}

	@Override
	public int describeContents() {
		return hashCode();
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (!(obj instanceof ParcelableDirectMessage)) return false;
		final ParcelableDirectMessage other = (ParcelableDirectMessage) obj;
		if (account_id != other.account_id) return false;
		if (id != other.id) return false;
		return true;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (int) (account_id ^ account_id >>> 32);
		result = prime * result + (int) (id ^ id >>> 32);
		return result;
	}

	@Override
	public String toString() {
		return "ParcelableDirectMessage{account_id=" + account_id + ", id=" + id + ", timestamp=" + timestamp
				+ ", sender_id=" + sender_id + ", recipient_id=" + recipient_id + ", is_outgoing=" + is_outgoing
				+ ", text_html=" + text_html + ", text_plain=" + text_plain + ", text_unescaped=" + text_unescaped
				+ ", sender_name=" + sender_name + ", recipient_name=" + recipient_name + ", sender_screen_name="
				+ sender_screen_name + ", recipient_screen_name=" + recipient_screen_name
				+ ", sender_profile_image_url=" + sender_profile_image_url + ", recipient_profile_image_url="
				+ recipient_profile_image_url + "}";
	}

	@Override
	public void writeToParcel(final Parcel out, final int flags) {
		out.writeLong(account_id);
		out.writeLong(id);
		out.writeLong(timestamp);
		out.writeLong(sender_id);
		out.writeLong(recipient_id);
		out.writeInt(is_outgoing ? 1 : 0);
		out.writeString(text_html);
		out.writeString(text_plain);
		out.writeString(sender_name);
		out.writeString(recipient_name);
		out.writeString(sender_screen_name);
		out.writeString(recipient_screen_name);
		out.writeString(sender_profile_image_url);
		out.writeString(recipient_profile_image_url);
	}

}
