/*
 * 				Twidere - Twitter client for Android
 * 
 *  Copyright (C) 2012-2014 Mariotaku Lee <mariotaku.lee@gmail.com>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.mariotaku.twidere.model;

import java.util.Arrays;
import java.util.Comparator;

import android.os.Parcel;
import android.os.Parcelable;

public class ParcelableStatus implements Parcelable, Comparable<ParcelableStatus> {

	public static final Parcelable.Creator<ParcelableStatus> CREATOR = new Parcelable.Creator<ParcelableStatus>() {
		@Override
		public ParcelableStatus createFromParcel(final Parcel in) {
			return new ParcelableStatus(in);
		}

		@Override
		public ParcelableStatus[] newArray(final int size) {
			return new ParcelableStatus[size];
		}
	};

	public static final Comparator<ParcelableStatus> TIMESTAMP_COMPARATOR = new Comparator<ParcelableStatus>() {

		@Override
		public int compare(final ParcelableStatus object1, final ParcelableStatus object2) {
			final long diff = object2.timestamp - object1.timestamp;
			if (diff > Integer.MAX_VALUE) return Integer.MAX_VALUE;
			if (diff < Integer.MIN_VALUE) return Integer.MIN_VALUE;
			return (int) diff;
		}
	};

	public static final Comparator<ParcelableStatus> REVERSE_ID_COMPARATOR = new Comparator<ParcelableStatus>() {

		@Override
		public int compare(final ParcelableStatus object1, final ParcelableStatus object2) {
			final long diff = object1.id - object2.id;
			if (diff > Integer.MAX_VALUE) return Integer.MAX_VALUE;
			if (diff < Integer.MIN_VALUE) return Integer.MIN_VALUE;
			return (int) diff;
		}
	};

	public final long retweet_id, retweeted_by_id, id, account_id, user_id, timestamp, retweet_count,
			in_reply_to_status_id, in_reply_to_user_id, my_retweet_id;

	public final boolean is_gap, is_retweet, is_favorite, has_media, is_possibly_sensitive, user_is_following,
			user_is_protected, user_is_verified;

	public final String retweeted_by_name, retweeted_by_screen_name, text_html, text_plain, user_name,
			user_screen_name, in_reply_to_name, in_reply_to_screen_name, source, user_profile_image_url, media_link,
			text_unescaped;

	public final ParcelableLocation location;

	public final ParcelableUserMention[] mentions;

	public ParcelableStatus(final Parcel in) {
		retweet_id = in.readLong();
		retweeted_by_id = in.readLong();
		id = in.readLong();
		account_id = in.readLong();
		user_id = in.readLong();
		timestamp = in.readLong();
		retweet_count = in.readLong();
		in_reply_to_status_id = in.readLong();
		is_gap = in.readInt() == 1;
		is_retweet = in.readInt() == 1;
		is_favorite = in.readInt() == 1;
		user_is_protected = in.readInt() == 1;
		user_is_verified = in.readInt() == 1;
		has_media = in.readInt() == 1;
		retweeted_by_name = in.readString();
		retweeted_by_screen_name = in.readString();
		text_html = in.readString();
		text_plain = in.readString();
		user_name = in.readString();
		user_screen_name = in.readString();
		in_reply_to_screen_name = in.readString();
		source = in.readString();
		user_profile_image_url = in.readString();
		media_link = in.readString();
		location = in.readParcelable(ParcelableLocation.class.getClassLoader());
		my_retweet_id = in.readLong();
		is_possibly_sensitive = in.readInt() == 1;
		user_is_following = in.readInt() == 1;
		text_unescaped = in.readString();
		in_reply_to_user_id = in.readLong();
		in_reply_to_name = in.readString();
		final Parcelable[] mentions_parcelable = in.readParcelableArray(ParcelableUserMention.class.getClassLoader());
		mentions = mentions_parcelable != null ? ParcelableUserMention.CREATOR.newArray(mentions_parcelable.length)
				: null;
		if (mentions_parcelable != null) {
			for (int i = 0, j = mentions_parcelable.length; i < j; i++) {
				mentions[i] = (ParcelableUserMention) mentions_parcelable[i];
			}
		}
	}

	@Override
	public int compareTo(final ParcelableStatus another) {
		if (another == null) return 0;
		final long diff = another.id - id;
		if (diff > Integer.MAX_VALUE) return Integer.MAX_VALUE;
		if (diff < Integer.MIN_VALUE) return Integer.MIN_VALUE;
		return (int) diff;
	}

	@Override
	public int describeContents() {
		return 0;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (!(obj instanceof ParcelableStatus)) return false;
		final ParcelableStatus other = (ParcelableStatus) obj;
		if (account_id != other.account_id) return false;
		if (id != other.id) return false;
		return true;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (int) (account_id ^ account_id >>> 32);
		result = prime * result + (int) (id ^ id >>> 32);
		return result;
	}

	@Override
	public String toString() {
		return "ParcelableStatus{retweet_id=" + retweet_id + ", retweeted_by_id=" + retweeted_by_id + ", id=" + id
				+ ", account_id=" + account_id + ", user_id=" + user_id + ", timestamp=" + timestamp
				+ ", retweet_count=" + retweet_count + ", in_reply_to_status_id=" + in_reply_to_status_id
				+ ", in_reply_to_user_id=" + in_reply_to_user_id + ", my_retweet_id=" + my_retweet_id + ", is_gap="
				+ is_gap + ", is_retweet=" + is_retweet + ", is_favorite=" + is_favorite + ", has_media=" + has_media
				+ ", is_possibly_sensitive=" + is_possibly_sensitive + ", user_is_following=" + user_is_following
				+ ", user_is_protected=" + user_is_protected + ", user_is_verified=" + user_is_verified
				+ ", retweeted_by_name=" + retweeted_by_name + ", retweeted_by_screen_name=" + retweeted_by_screen_name
				+ ", text_html=" + text_html + ", text_plain=" + text_plain + ", user_name=" + user_name
				+ ", user_screen_name=" + user_screen_name + ", in_reply_to_name=" + in_reply_to_name
				+ ", in_reply_to_screen_name=" + in_reply_to_screen_name + ", source=" + source
				+ ", user_profile_image_url=" + user_profile_image_url + ", media_link=" + media_link
				+ ", text_unescaped=" + text_unescaped + ", location=" + location + ", mentions="
				+ Arrays.toString(mentions) + "}";
	}

	@Override
	public void writeToParcel(final Parcel out, final int flags) {
		out.writeLong(retweet_id);
		out.writeLong(retweeted_by_id);
		out.writeLong(id);
		out.writeLong(account_id);
		out.writeLong(user_id);
		out.writeLong(timestamp);
		out.writeLong(retweet_count);
		out.writeLong(in_reply_to_status_id);
		out.writeInt(is_gap ? 1 : 0);
		out.writeInt(is_retweet ? 1 : 0);
		out.writeInt(is_favorite ? 1 : 0);
		out.writeInt(user_is_protected ? 1 : 0);
		out.writeInt(user_is_verified ? 1 : 0);
		out.writeInt(has_media ? 1 : 0);
		out.writeString(retweeted_by_name);
		out.writeString(retweeted_by_screen_name);
		out.writeString(text_html);
		out.writeString(text_plain);
		out.writeString(user_name);
		out.writeString(user_screen_name);
		out.writeString(in_reply_to_screen_name);
		out.writeString(source);
		out.writeString(user_profile_image_url);
		out.writeString(media_link);
		out.writeParcelable(location, flags);
		out.writeLong(my_retweet_id);
		out.writeInt(is_possibly_sensitive ? 1 : 0);
		out.writeInt(user_is_following ? 1 : 0);
		out.writeString(text_unescaped);
		out.writeLong(in_reply_to_user_id);
		out.writeString(in_reply_to_name);
		out.writeParcelableArray(mentions, flags);
	}

}
