/*
 * 				Twidere - Twitter client for Android
 * 
 *  Copyright (C) 2012-2014 Mariotaku Lee <mariotaku.lee@gmail.com>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.mariotaku.twidere.model;

import android.os.Parcel;
import android.os.Parcelable;

public class ParcelableUser implements Parcelable, Comparable<ParcelableUser> {

	public static final Parcelable.Creator<ParcelableUser> CREATOR = new Parcelable.Creator<ParcelableUser>() {
		@Override
		public ParcelableUser createFromParcel(final Parcel in) {
			return new ParcelableUser(in);
		}

		@Override
		public ParcelableUser[] newArray(final int size) {
			return new ParcelableUser[size];
		}
	};

	public final long account_id, id, created_at, position;

	public final boolean is_protected, is_verified, is_follow_request_sent, is_following;

	public final String description_plain, name, screen_name, location, profile_image_url, profile_banner_url, url,
			url_expanded, description_html, description_unescaped, description_expanded;

	public final int followers_count, friends_count, statuses_count, favorites_count;

	public final boolean is_cache;

	public ParcelableUser(final Parcel in) {
		position = in.readLong();
		account_id = in.readLong();
		id = in.readLong();
		created_at = in.readLong();
		is_protected = in.readInt() == 1;
		is_verified = in.readInt() == 1;
		name = in.readString();
		screen_name = in.readString();
		description_plain = in.readString();
		location = in.readString();
		profile_image_url = in.readString();
		profile_banner_url = in.readString();
		url = in.readString();
		is_follow_request_sent = in.readInt() == 1;
		followers_count = in.readInt();
		friends_count = in.readInt();
		statuses_count = in.readInt();
		favorites_count = in.readInt();
		is_cache = in.readInt() == 1;
		description_html = in.readString();
		description_expanded = in.readString();
		url_expanded = in.readString();
		is_following = in.readInt() == 1;
		description_unescaped = in.readString();
	}

	@Override
	public int compareTo(final ParcelableUser that) {
		final long diff = that != null ? position - that.position : position;
		if (diff > Integer.MAX_VALUE) return Integer.MAX_VALUE;
		if (diff < Integer.MIN_VALUE) return Integer.MIN_VALUE;
		return (int) diff;
	}

	@Override
	public int describeContents() {
		return 0;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (!(obj instanceof ParcelableUser)) return false;
		final ParcelableUser other = (ParcelableUser) obj;
		if (account_id != other.account_id) return false;
		if (id != other.id) return false;
		return true;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (int) (account_id ^ account_id >>> 32);
		result = prime * result + (int) (id ^ id >>> 32);
		return result;
	}

	@Override
	public String toString() {
		return "ParcelableUser{account_id=" + account_id + ", id=" + id + ", created_at=" + created_at + ", position="
				+ position + ", is_protected=" + is_protected + ", is_verified=" + is_verified
				+ ", is_follow_request_sent=" + is_follow_request_sent + ", is_following=" + is_following
				+ ", description_plain=" + description_plain + ", name=" + name + ", screen_name=" + screen_name
				+ ", location=" + location + ", profile_image_url=" + profile_image_url + ", profile_banner_url="
				+ profile_banner_url + ", url=" + url + ", url_expanded=" + url_expanded + ", description_html="
				+ description_html + ", description_unescaped=" + description_unescaped + ", description_expanded="
				+ description_expanded + ", followers_count=" + followers_count + ", friends_count=" + friends_count
				+ ", statuses_count=" + statuses_count + ", favorites_count=" + favorites_count + ", is_cache="
				+ is_cache + "}";
	}

	@Override
	public void writeToParcel(final Parcel out, final int flags) {
		out.writeLong(position);
		out.writeLong(account_id);
		out.writeLong(id);
		out.writeLong(created_at);
		out.writeInt(is_protected ? 1 : 0);
		out.writeInt(is_verified ? 1 : 0);
		out.writeString(name);
		out.writeString(screen_name);
		out.writeString(description_plain);
		out.writeString(location);
		out.writeString(profile_image_url);
		out.writeString(profile_banner_url);
		out.writeString(url);
		out.writeInt(is_follow_request_sent ? 1 : 0);
		out.writeInt(followers_count);
		out.writeInt(friends_count);
		out.writeInt(statuses_count);
		out.writeInt(favorites_count);
		out.writeInt(is_cache ? 1 : 0);
		out.writeString(description_html);
		out.writeString(description_expanded);
		out.writeString(url_expanded);
		out.writeInt(is_following ? 1 : 0);
		out.writeString(description_unescaped);
	}

}
